/* Switch modes of association depending on the availability of Infrastructure or lack \
*  thereof.
*
*  Platform: Linux
*  Hardware: Wireless interface up
*  Required: Wireless Tools for Linux 
*  
*  If you run the 'orinoco_cs' driver, make sure the firmware is > 6.16 to support the \
*  "IBSS Ad-Hoc" mode, and scanning for AP's using iwlist. You can find out what       \
*  firmware version is on the card by checking your 'dmesg' output when the card       \
*  interface is brought up.
*
*  Firmware upgrades at: http://airsnort.shmoo.com/orinocoinfo.html 
*       Yes, you will have to flash it on a win32 machine, as firmware upgrades aren't \
*       available for Linux yet.
*
*  Starting reference: http://www.hpl.hp.com/personal/Jean_Tourrilhes/Linux/
* 
*/


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <sys/time.h>
#include <time.h>

//#define SNR(x,y) 10*log10(x/y)
#define SNR(signal,noise) 	signal - noise  // assumption

// const
#define MAXLEN		80
#define MAXCMDLEN	150
//#define MINSNR	30
#define GRACE		5
#define ADHOCESSID	"7DS"
#define ADHOCWINDOW	5
#define IFSIZE		10
#define DELAY		2
#define	SWEEPNR		2
#define CELLNET		" 192.168.47."

// commands
#define IWCONFIG	"iwconfig "
#define IWLIST		"iwlist "
#define IFCONFIG	"ifconfig "
#define DHCLIENT	"dhclient "

struct iwliststat {
	unsigned int nr;
	float	freq;
	char 	hw_addr[18],
		essid [80],
		mode [25],
		encryption[5];
		
	int 	quality,
		signal_level,
		noise_level,
		snr;
		
	struct 	iwliststat *next;
};

struct iwconfigstat {
	char 	essid [80],
		mode[25],
		hw_addr[18];
};

struct packets
{
	int 	rxpackets,
		txpackets;
};

// global
int 	minsnr,
	nullscancount,
	sweepcount = 0,
	infrastructure_present = 0,
	adhoc_present = 0,
	lastadhoc = 0,
	lastinfrastructure = 0,
	ip_addr = 0,	// last used ip_addr in ADHOCESSID
	flag = 0;	// for calling dhclient only once as iwconfig is called twice

char 	last_adhoc_ip_addr[4] = {'\0'},		// last used ip_addr in ad-hoc
	ifconfigdir[MAXCMDLEN],
	iwconfigdir[MAXCMDLEN],
	iwlistdir[MAXCMDLEN];
	//iwdir[MAXCMDLEN];

struct iwliststat *liwlist;

// prototypes
void	usage (char *pgm_name);
struct	iwliststat * iwlist (char * cmd);
struct	iwconfigstat * iwconfig (char * cmd);
void	iwconfigswitchto (char * dev, char * mode, char * essid);
char *  getvalueof (char *parameter, char *line, char *begin, char *end);
void	printiwlist (struct iwliststat * liwlist);
void	printiwconfig (struct iwconfigstat * liwconfig);
void	deleteiwlist (struct iwliststat * liwlist);
struct 	packets * datatransfer (char * cmd);

int
main ( argc, argv )
	int argc;
	char ** argv;
{
	char 	dev[IFSIZE],
		iwlistcmd[MAXCMDLEN], 
		iwconfigcmd[MAXCMDLEN], 
		ifconfigcmd[MAXCMDLEN],
		* lslash;
	
	liwlist   = (struct iwliststat *) malloc (sizeof (struct iwliststat));
	struct iwconfigstat  *liwconfig = (struct iwconfigstat *) malloc (sizeof (struct iwconfigstat));
	
	FILE *fp;
	
	int 	avgsnr,
		sumsnr,
		ret;

	if (argc != 3)
		usage (argv[0]);
	
	// get minsnr;
	minsnr = (int) atoi (argv[2]);
	
	// get interface name into 'dev'
	strncpy (dev, argv[1], IFSIZE);

	// get dir locations of commands
	if ((fp = popen ("which ifconfig", "r")) == 0) {
		perror ("check ifconfig\n");
		exit (1);
	}

	while ((fgets (ifconfigdir, MAXLEN, fp)) != NULL) { }
	pclose (fp);

	if ((lslash = strrchr (ifconfigdir, '/')) == NULL) { perror ("strrchr"); exit (1); }
	lslash++;
	while (*lslash != '\0') {
		*lslash = '\0';
		lslash++;
	}

	if ((fp = popen ("which iwconfig", "r")) == 0) {
		perror ("check iwconfig\n");
		exit (1);
	}

	while ((fgets (iwconfigdir, MAXLEN, fp)) != NULL) { }
	pclose (fp);

	if ((lslash = strrchr (iwconfigdir, '/')) == NULL) { perror ("strrchr"); exit (1); }
	lslash++;
	while (*lslash != '\0') {
		*lslash = '\0';
		lslash++;
	}

	if ((fp = popen ("which iwlist", "r")) == 0) {
		perror ("check iwlist\n");
		exit (1);
	}

	while ((fgets (iwlistdir, MAXLEN, fp)) != NULL) { }
	pclose (fp);

	if ((lslash = strrchr (iwlistdir, '/')) == NULL) { perror ("strrchr"); exit (1); }
	lslash++;
	while (*lslash != '\0') {
		*lslash = '\0';
		lslash++;
	}

	// iwlist command
	strncpy (iwlistcmd, iwlistdir, MAXCMDLEN);
	strncat (iwlistcmd, IWLIST, MAXCMDLEN);
	strncat (iwlistcmd, dev, IFSIZE);
	strncat (iwlistcmd, " scan 2>/dev/null", strlen (" scan 2>/dev/null"));

	// iwconfig command
	strncpy (iwconfigcmd, iwconfigdir, MAXCMDLEN);
	strncat (iwconfigcmd, IWCONFIG, MAXCMDLEN);
	strncat (iwconfigcmd, dev, IFSIZE);
	strncat (iwconfigcmd, " 2>/dev/null", strlen (" 2>/dev/null"));

	// ifconfig command
	strncpy (ifconfigcmd, ifconfigdir, MAXCMDLEN);
	strncat (ifconfigcmd, IFCONFIG, MAXCMDLEN);
	strncat (ifconfigcmd, dev, IFSIZE);
	strncat (ifconfigcmd, " 2>/dev/null", strlen (" 2>/dev/null"));

	while (1) {
		system ("clear");

		printf ("Min SNR : %d\n", minsnr);
		// To scan while in adhoc, use essid 'any' in iwconfig. 
		// This will mean the association would drop,
		// and we will have to bring it up back again if we dont 
		// get another AP to connect to.

		// use iwconfig to get the current mode
		liwconfig = iwconfig (iwconfigcmd);

		// in infrastructure
		if ((ret = strncmp (liwconfig->mode, "Ad-Hoc", strlen ("Ad-Hoc"))) != 0) {	
			printf ("Mode:%s \tESSID :%s.\n", liwconfig->mode, liwconfig->essid);
			liwlist = iwlist (iwlistcmd);
			
			// check if snr is below the MINSNR level
			if (liwlist != NULL) {
				if (liwlist->snr < minsnr) {
					printf ("SNR falling below %d \t Scan %d...\n", minsnr, sweepcount + 1);
					if ((sweepcount++ > SWEEPNR) && (avgsnr < minsnr)) {
						printf ("SNR fell below %d: switch to ADHOC now...\n", minsnr);
						flag = 0;
						// no check to see if data is being transferred
						iwconfigswitchto (dev, "Ad-Hoc", ADHOCESSID);
						//iwconfigswitchto (dev, "Ad-Hoc", ADHOCESSID);
						//sleep (DELAY);
						sweepcount = 0;
						sumsnr = 0;
						avgsnr = 0;
					}
					else {
						sumsnr += liwlist->snr;
						avgsnr = sumsnr/sweepcount;
					}
				}
				else {
					printf ("SNR within bounds\n");	
					sweepcount = 0;
					sumsnr = avgsnr = 0;
				}
			}
		}

		// in ad-hoc
		if ((ret = strncmp (liwconfig->mode, "Ad-Hoc", strlen ("Ad-Hoc"))) == 0) {	
			printf ("Mode: %s \tESSID :%s.\n", liwconfig->mode, liwconfig->essid);
	
			// check if we need to switch to essid 'any' to run 'iwlist scan'
			if ((ret = strncmp (liwconfig->essid, ADHOCESSID, strlen (ADHOCESSID))) == 0) {
				printf ("Connected to %s in Ad-Hoc\nSwitching to 'any' to scan...\n", ADHOCESSID);
				flag = 1;
				// sleep here so we don't keep switching back n forth too often
				sleep (DELAY);
	
				iwconfigswitchto (dev, "Ad-Hoc", "any");
				printf ("Switched to essid 'any' ... getting scan list ... \n");
				liwlist = iwlist (iwlistcmd);

				// got scan results
				// do we see a 'good' AP in Infratructure?
				if ((liwlist != NULL) && ((ret = strncmp (liwlist->encryption, "off", strlen ("off")) == 0))) {
					if ((ret = strncmp (liwlist->mode, "Ad-Hoc", strlen ("Ad-Hoc"))) != 0) {
						if (liwlist->snr >= (minsnr + GRACE)) {
							printf ("Switching from Ad-Hoc to Infrastructure on :%s.\n", liwlist->essid);
							flag = 0;
							iwconfigswitchto (dev, "Managed", liwlist->essid);
							//iwconfigswitchto (dev, "Managed", liwlist->essid);
							
							printf ("Switch to infrastructure from ad-hoc complete\n");
							sweepcount = 0;

						}
						if (liwlist->snr >= minsnr) {
							printf ("In Ad-Hoc...\tInfrastructure AP with good SNR seen, and ENCR: off\n");
							if ((sweepcount++ > SWEEPNR) && (avgsnr >= minsnr)) {
								printf ("Switching from Ad-Hoc to Infrastructure on :%s.\n", liwlist->essid);
								flag = 0;
								iwconfigswitchto (dev, "Managed", liwlist->essid);
								//iwconfigswitchto (dev, "Managed", liwlist->essid);
								
								printf ("Switch to infrastructure from ad-hoc complete\n");
								sweepcount = 0;
								sumsnr = 0;
								avgsnr = 0;
							}
							else {
								sumsnr += liwlist->snr;
								avgsnr = sumsnr/sweepcount;
							}
						}
						else {
							printf ("Checking snr of AP :%s. Revert to %s.\n", liwlist->essid, ADHOCESSID);
							flag = 1;
							iwconfigswitchto (dev, "Ad-Hoc", ADHOCESSID);
							//iwconfigswitchto (dev, "Ad-Hoc", ADHOCESSID);
							//sweepcount = 0;
						}
					}
				}
	
				// no infrastructure AP, switch back to ADHOCESSID
				else {
					printf ("No AP in infrastructure seen. Revert to Ad-Hoc on :%s.\n", ADHOCESSID);
					flag = 1;
					iwconfigswitchto (dev, "Ad-Hoc", ADHOCESSID);
					//iwconfigswitchto (dev, "Ad-Hoc", ADHOCESSID);
					sweepcount = 0;
				}
					
			}
			else {
				printf ("In Ad-Hoc mode but not on :%s.\t Reverting ...\n", ADHOCESSID);
				liwlist = iwlist (iwlistcmd);
				flag = 1;
				iwconfigswitchto (dev, "Ad-Hoc", ADHOCESSID);
				//iwconfigswitchto (dev, "Ad-Hoc", ADHOCESSID);
				sleep (1);
			}

		}
		
		// not connected to any AP
		if ((liwconfig->mode == NULL) || (liwconfig->essid == NULL)) {	
			printf ("Not connected to any AP in any mode \t Scan %d...\n", sweepcount);
			if (sweepcount++ > SWEEPNR) {
				printf ("Not associated to any AP, switch to ADHOCESSID\n");
				flag = 0;
				iwconfigswitchto (dev, "Ad-Hoc", ADHOCESSID);
				//iwconfigswitchto (dev, "Ad-Hoc", ADHOCESSID);
				sweepcount = 0;
			}
			printf ("ScanCount: %d\n", sweepcount);
		}
	
		//printiwconfig (liwconfig);
		
		if (liwlist != NULL) {
			printiwlist (liwlist);
			deleteiwlist (liwlist);
		}
		else {
			//printf ("No APs found in scan :%d.\n", sweepcount);
			// NO scan results for SWEEPNR times
			if (sweepcount++ > SWEEPNR) {
				printf ("No good APs found in %d scans\n", sweepcount + 1);
				flag = 0;
				iwconfigswitchto (dev, "Ad-Hoc", ADHOCESSID);
				//iwconfigswitchto (dev, "Ad-Hoc", ADHOCESSID);
				sweepcount = 0;
			}
		}

		free (liwconfig);
		infrastructure_present = adhoc_present = 0;
		sleep (DELAY);
	}
	
	return 0;
}
// main()

struct iwliststat *
iwlist ( cmd )
	char cmd[MAXCMDLEN];
{
	int 	i, j, k, 
		count, ret,
		t1, t2;

	FILE 	*fp;	
	char 	line[100][MAXLEN + 1], 
		tmp[80],
		*cstart;
	
	if ( (fp = popen (cmd, "r")) == 0) {
		perror ("iwlist");
		exit (1);
	}	
	
	count = 0;
	// read from pipe
	while ( (fgets (line[count], MAXLEN, fp)) != NULL )
		count++;
	
	pclose (fp);

	// if we have APs in range
	if ( count > 1 ) {
		struct iwliststat *list = (struct iwliststat *) malloc (sizeof (struct iwliststat));
		struct iwliststat *p = list;

		// assuming 6 lines here for iwlist scan results for a particular cell
		for (i = 1, k = 1; i < count - 1; i += 6, k++, p->next = (struct iwliststat *) malloc (sizeof (struct iwliststat)), p = p->next, p->next = NULL) {
			p->nr = k;
		
			// get HW Addr			
			if ((cstart = strstr (line[i], "Address:")) == NULL) {
				perror ("IWLIST: HW Address not found");
				exit (1);
			}
			cstart += sizeof ("Address:");

			j = 0;
			while (*cstart != '\n') {
				p->hw_addr[j] = *cstart;
				j++;
				cstart++;
			}

			p->hw_addr[j] = '\0';
			
			// get ESSID
			if ((cstart = strstr (line[i + 1], "ESSID:")) == NULL) {
				perror ("IWLIST: ESSID not found");
				exit (1);
			}
			cstart += sizeof ("ESSID:");

			j = 0;
			while (*cstart != '"') {
				p->essid[j] = *cstart;
				j++;
				cstart++;
			}

			p->essid[j] = '\0';

			// get Mode
			if ((cstart = strstr (line[i + 2], "Mode:")) == NULL) {
				perror ("IWLIST: Mode not found");
				exit (1);
			}
			cstart += sizeof ("Mode");

			j = 0;
			while (*cstart != '\n') {
				p->mode[j] = *cstart;
				j++;
				cstart++;
			}

			p->mode[j] = '\0';

			if ( (ret = strncmp (p->mode, "Master", strlen ("Master"))) == 0 || (ret = strncmp (p->mode, "Managed", strlen ("Managed"))) == 0 )
				infrastructure_present = 1;
			if ( (ret = strncmp (p->mode, "Ad-Hoc", strlen ("Ad-Hoc"))) == 0)
				adhoc_present = 1;
	
			// get frequency
			if ((cstart = strstr (line[i + 3], "Frequency:")) == NULL) {
				perror ("IWLIST: Frequency not found");
				exit (1);
			}
			cstart += sizeof ("Frequency");

			j = 0;
			while (*cstart != 'G') {
				tmp[j] = *cstart;
				j++;
				cstart++;
			}

			tmp[j] = '\0';
			p->freq = atof (tmp);

			// get quality
			if ((cstart = strstr (line[i + 4], "Quality:")) == NULL) {
				perror ("IWLIST: Quality not found");
				exit (1);
			}
			cstart += sizeof ("Quality");

			j = 0;
			while (*cstart != '/') {
				tmp[j] = *cstart;
				j++;
				cstart++;
			}

			tmp[j] = '\0';
			t1 = atof (tmp);

			j = 0;
			while (*cstart != ' ') {
				tmp[j] = *cstart;
				j++;
				cstart++;
			}

			tmp[j] = '\0';
			t2 = atof (tmp);

			if (t2 != 0)
				p->quality = t1/t2;
			else p->quality = -1;

			// get signal level 
			if ((cstart = strstr (line[i + 4], "Signal level:")) == NULL) {
				perror ("IWLIST: Signal level not found");
				exit (1);
			}
			cstart += sizeof ("Signal level");

			j = 0;
			while (*cstart != ' ') {
				tmp[j] = *cstart;
				j++;
				cstart++;
			}

			tmp[j] = '\0';
			p->signal_level = atoi (tmp);

			// get noise level 
			if ((cstart = strstr (line[i + 4], "Noise level:")) == NULL) {
				perror ("IWLIST: Noise level not found");
				exit (1);
			}
			cstart += sizeof ("Noise level");

			j = 0;
			while (*cstart != ' ') {
				tmp[j] = *cstart;
				j++;
				cstart++;
			}

			tmp[j] = '\0';
			p->noise_level = atoi (tmp);

			// get SNR
			p->snr = SNR (p->signal_level, p->noise_level);
		
			// get encryption
			if ((cstart = strstr (line[i + 5], "Encryption key:")) == NULL) {
				perror ("IWLIST: Encryption key not found");
				exit (1);
			}
			cstart += sizeof ("Encryption key");

			j = 0;
			while (*cstart != '\n') {
				p->encryption[j] = *cstart;
				j++;
				cstart++;
			}

			p->encryption[j] = '\0';

		}
		
		return list;
	}
	
	else 
		return NULL;
	
}
// iwliststat()

struct iwconfigstat *
iwconfig (cmd)
	char cmd[MAXCMDLEN];
{
	int 	count, 
		ret,
		j; 

	struct 	iwconfigstat * p; 
	p = (struct iwconfigstat *)malloc (sizeof (struct iwconfigstat));

	FILE 	*fp;
	
	char 	line[2][MAXLEN + 1],
		*cstart;

	if ( (fp = popen (cmd, "r")) == 0 ) {
		perror ("iwconfig");
	}	
		
	// read from pipe - get the line for the MAC addr
	for (count = 0; count < 2; count++) 
		fgets (line[count], MAXLEN, fp);
	
	pclose (fp);

	if (count > 1) {
		// get essid of AP
		if ((cstart = strstr (line[0], "ESSID:")) == NULL) {
			perror ("IWCONFIG: ESSID not found");
			exit (1);
		}
		cstart += sizeof ("ESSID:");

		j = 0;
		while (*cstart != '"') {
			p->essid[j] = *cstart;
			j++;
			cstart++;
		}

		p->essid[j] = '\0';

		// get Mode
		if ((cstart = strstr (line[1], "Mode:")) == NULL) {
			perror ("IWCONFIG: Mode not found");
			exit (1);
		}
		cstart += sizeof ("Mode");

		j = 0;
		while (*cstart != ' ') {
			p->mode[j] = *cstart;
			j++;
			cstart++;
		}

		p->mode[j] = '\0';

		// get the MAC
		if ( (ret = strncmp (p->mode, "Ad-Hoc", strlen ("Ad-Hoc"))) == 0) {
			if ((cstart = strstr (line[1], "Cell:")) == NULL) {
				perror ("IWCONFIG: HW Address not found");
				exit (1);
			}

			cstart += sizeof ("Cell:");
		}
		else {
			if ((cstart = strstr (line[1], "Point:")) == NULL) {
				perror ("IWCONFIG: HW Address not found");
				exit (1);
			}

			cstart += sizeof ("Point:");
		}

		j = 0;
		while (*cstart != '\n') {
			p->hw_addr[j] = *cstart;
			j++;
			cstart++;
		}

		p->hw_addr[j] = '\0';
	
		return p;
	}
	else return NULL;
}
// iwconfig ()

void
iwconfigswitchto (dev, mode, essid)
	char 	dev[IFSIZE],
		mode[MAXLEN],
		essid[MAXLEN];
{
	char 	iwcmd[MAXCMDLEN],
		ifcmd[MAXCMDLEN],
		last_bits[3],
		dhcmd[MAXLEN];
	
	int 	cell_found = 0,
		ret;

	float time_taken;
		
	struct iwliststat *p;

	p = liwlist;
	
	struct timeval	tstart, 
			tend;

	strncpy (iwcmd, iwconfigdir, MAXCMDLEN);
	strncat (iwcmd, IWCONFIG, MAXCMDLEN);
	strncat (iwcmd, dev, strlen (dev));
	strncat (iwcmd, " mode \"", strlen (" mode \""));
	strncat (iwcmd, mode, strlen (mode));
	strncat (iwcmd, "\" essid \"", strlen ("\" essid \""));
	strncat (iwcmd, essid, strlen (essid));
	strncat (iwcmd, "\" 2>/dev/null", strlen ("\" 2>/dev/null"));

	strncpy (dhcmd, DHCLIENT, MAXLEN);
	strncat (dhcmd, dev, strlen (dev));
	
	gettimeofday (&tstart, NULL);
	system (iwcmd);
	gettimeofday (&tend, NULL);
	time_taken = ((float)tend.tv_usec - (float)tstart.tv_usec); 

	if (time_taken > 0)
		printf ("iwconfig took : %0.3f msecs\n", (float)(time_taken / 1000));

	// moving into Infrastructure, use DHCP to assign IP addr
	if ( (flag == 0) && ((ret = strncmp (mode, "Ad-Hoc", strlen ("Ad-Hoc"))) != 0) ) {
		system (dhcmd);
		flag = 1;
	}
	
	// moving into Ad-Hoc, assign an IP addr
	else if ((ret = strncmp (essid, "any", strlen ("any"))) != 0){
		strncpy (ifcmd, ifconfigdir, MAXCMDLEN);
		strncat (ifcmd, IFCONFIG, MAXCMDLEN);
		strncat (ifcmd, dev, strlen (dev));
		strncat (ifcmd, CELLNET, strlen (CELLNET));

		// search iwlist to see if there are any cells on ADHOCESSID
		if ( p != NULL ) {
			if ( p->next == NULL ) {
				if ((ret = strncmp (p->essid, ADHOCESSID, strlen (ADHOCESSID))) == 0) 
					cell_found++;
			}
			else {
				while (p->next != NULL) {
					if ((ret = strncmp (p->essid, ADHOCESSID, strlen (ADHOCESSID))) == 0) 
						cell_found++;

					p = p->next;
				}
			}
		}
	
		printf ("No of cells found on %s = %d\n", ADHOCESSID, cell_found);
		if ((cell_found > 0) && (ip_addr == 0)) 
			ip_addr = cell_found * 2;	// random multiple, this

		if ((cell_found > 0) && (ip_addr != 0)) {
			if ((ip_addr < (cell_found * 2)))
				ip_addr = cell_found * 2;
			else {}		// use the last assigned ip_addr
		}

		if ((cell_found == 0) && (ip_addr == 0)) 
			ip_addr = 1;
		
		if ((cell_found == 0) && (ip_addr != 0)) { }	// use the last assigned ip_addr
	
		sprintf (last_bits, "%i", ip_addr);
		strncpy (last_adhoc_ip_addr, last_bits, strlen (last_bits));
	
		strncat (ifcmd, last_bits, strlen (last_bits));	

		system (ifcmd);

		printf ("IP addr:\t%s%s\n", CELLNET, last_bits);
	
	}
	sleep (DELAY);
} 
// iwconfigswitchto ()

char *
getvalueof (element, line, start, stop)
	char 	*element,
		*line,
		*start, *stop;
{
	char *val;
	
	return val;
}
		
		

struct packets *
datatransfer (cmd)
	char cmd[MAXCMDLEN];
{
	struct packets *p;
	FILE 	*fp;
	char	line[15][MAXLEN + 1],
		tmp[20],
		*cstart;
	
	int 	lncount = 0, 
		i,
		rxpackets = 0,
		txpackets = 0;

	if ((fp = popen (cmd, "r")) == NULL) {
		printf ("ifconfig failed\n");
		exit (1);
	}

	while (fgets (line[lncount], MAXLEN, fp) != NULL)
		lncount++;
	
	pclose (fp);
	
	if ((cstart = strstr (line[3], "RX packets:")) == NULL) {
		perror ("RX packets not found");
		exit (1);
	}
	cstart += sizeof ("RX packets");

	i = 0;
	while (*cstart != ' ') {
		tmp[i] = *cstart;
		i++;
		cstart++;
	}

	tmp[i] = '\0';
	rxpackets = atoi (tmp);
	
	if ((cstart = strstr (line[4], "TX packets:")) == NULL) {
		perror ("TX packets not found");
		exit (1);
	}
	cstart += sizeof ("TX packets");

	i = 0;
	while (*cstart != ' ') {
		tmp[i] = *cstart;
		i++;
		cstart++;
	}

	tmp[i] = '\0';
	txpackets = atoi (tmp);

	p->rxpackets = rxpackets;
	p->txpackets = txpackets;
	
	return p;
}
// datatransfer ()

void
printiwlist (list)
	struct iwliststat *list;
{
	struct iwliststat *p;
	
	printf ("\n\nNR\tMAC Addr\t\tMode\tFreq\tQual\tSignal\tNoise\tSNR\tEncr\tESSID \n");
	for ( p = list; p->next != NULL; p = p->next ) 
		printf ("%2d\t%s\t%s\t%.3lf\t%4d\t%d\t%d\t%d\t%s\t%s\n", p->nr, p->hw_addr, p->mode, p->freq, p->quality, p->signal_level, p-> noise_level, p->snr,  p->encryption, p->essid);
}
// printiwlist()

void
printiwconfig (list)
	struct iwconfigstat *list;
{
	struct iwconfigstat *p = list;

	printf ("\nIWCONFIG\nMAC Addr\t\tMode\t\tESSID\n");
	printf ("%s\t%s\t%s\n\n", p->hw_addr, p->mode, p->essid);
}
// printiwconfig()

void
deleteiwlist (list)
	struct iwliststat * list;
{
	struct iwliststat *p, *next;
	
	for ( p = list; p != NULL; p = next) {
		next = p->next;
		free (p);
	}
}
// deleteiwlist()

void
usage (pgm_name)
	char * pgm_name;
{
	printf ("Usage: %s <interface> <min SNR>\n", pgm_name);
	exit (1);
}
