#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "httpd.h"

#define MAX_HOSTNAME 25
#define MAX_FILEBASE 80
#define MAX_DIRNAME 25

int usage (void);
int start_server (int, char **);

int
main (argc, argv)
	int argc;
	char **argv;
{
	int ret;

	if ( (ret = start_server (argc, argv)) != 0)
		printf ("Failed creating server\n");

	return 0;
}

int
start_server (argc, argv)
	int argc;
	char **argv;
{
	char host_name[ MAX_HOSTNAME + 1 ], 
	     file_base[ MAX_FILEBASE + 1 ],
	     dirname[ MAX_DIRNAME + 1 ],
	     *hname = host_name,
	     *fbase = file_base,
	     *dir = dirname,
	     *dirstart;

	// FILE *access, *err;
	
	unsigned int port;
	int len, i, ret;

	if (argc != 4)
	{
		usage();
		exit (1);
	}
	
	else 
	{
		strncpy (host_name, argv[1], MAX_HOSTNAME);  // get hostname
		
		port = atoi (argv[2]);  // get port number
		
		strncpy (file_base, argv[3], MAX_FILEBASE);  // get filebase

		// get dirname from filebase
		if ( (dirstart = strrchr (fbase, '/')) == NULL)
		{
			perror ("failed to get dir name");
			exit (1);
		}
		len = dirstart - fbase;

		for (i = 0; i < len; i++, dirstart++)
			dir[i] = *dirstart;	
		dir[i] = '\0';
	}
	
	// create server instance
	httpd * server;
	if ( (server = httpdCreate (hname, port)) == NULL)
	{
		perror ("Failed to create server instance\n");
		exit (1);
	}

	// set up logging
	/*
	if ((access = fopen ("access.log", "a+")) == NULL)
	{
		perror ("Failed to open activity log file");
		exit (1);
	}
	
	if ((err = fopen ("error.log", "a+")) == NULL)
	{
		perror ("Failed to open error log file");
		exit (1);
	}
	
	httpdSetAccessLog (server, access);
	httpdSetErrorLog (server, err);
	*/
	httpdSetAccessLog (server, stdout);
	httpdSetErrorLog (server, stdout);
	
	// set file base
	httpdSetFileBase (server, fbase);

	// define content available via the server
	httpdAddWildcardContent (server, dir, NULL, fbase);

	// set content type
	httpdSetContentType (server, "text/html");
	httpdSetContentType (server, "text/plain");
	httpdSetContentType (server, "text/rtf");
	
	httpdSetContentType (server, "image/gif");
	httpdSetContentType (server, "image/jpeg");
	httpdSetContentType (server, "image/png");
	httpdSetContentType (server, "image/tiff");
	
	httpdSetContentType (server, "application/pdf");
	httpdSetContentType (server, "application/postscript");
	httpdSetContentType (server, "application/csv");


	// poll for and process requests
	while (1)
	{
		if (i != 0)
		{
			printf("Listening on %s : port = %d\n\tfile base = %s\n\tdir = %s\n", hname, port, fbase, dir);
			i = 0;
		}
		
		if ((ret = httpdGetConnection (server, NULL)) < 0)
			continue;  // error geeting conn
		
		if (httpdReadRequest (server) < 0)
		{
			httpdEndRequest (server);
			continue;
		}

		httpdProcessRequest (server);
	
		httpdEndRequest (server);
	}
	
	/*
	if ((ret = fclose (access)) != 0)
		perror ("Failed to close activity log file");

	if ((ret = fclose (err)) != 0)
		perror ("Failed to close err log file");
	*/
	
	return 0;
}

int
usage (void)
{
	printf ("Usage: <host_name> <port_number> <file_base>\n");
	return 0;
}

